'use strict';

var reverend = require('reverend');

module.exports = function(Vue, page, utils) {
    var _ = Vue.util;

    Vue._getPathFromState = function(routes, stateName, routeParams, queryParams) {
        var state = routes[stateName];

        if (!state) {
            _.warn('v-view: failed to resolve state: ' + stateName);
            return;
        }

        routeParams || (routeParams = {});

        var url = state.url;
        var path = reverend(url, routeParams);

        if (routeParams && url === path && !queryParams) {
            queryParams = routeParams;
        }

        if (!utils.isEmptyObject(queryParams)) {
            path += utils.buildQuery(queryParams);
        }

        return path;
    }

    return {
        init: function() {
            this._root = this.vm.$root;
            this._router = this._root.$options.router;
            this._options = this._router.options || {};

            this._states = this._router.states;

            this._beforeLocation = null;

            if (this._states) {
                this._initStates();
            } else {
                _.warn('v-view: $root must have "states" option');
            }

            page.start(this._options);

            this._defaultRoute && this._initDefaultRoute();
        },

        _initStates: function() {
            var states = this._routesType = this._states;
            this._defaultRoute = states.default || null;
            delete states.default;

            Object.keys(states).forEach(function(name) {
                var state = states[name];
                state.name = name;

                this._addState(state);
            }.bind(this));

            Vue.go = function(stateName, routeParams, queryParams) {
                var path;

                // Vue.go 支持path跳转
                if (stateName.indexOf('http') != 0 && stateName.indexOf('/') != 0) {
                    path = Vue._getPathFromState(states, stateName, routeParams, queryParams);
                } else {
                    path = stateName;
                }

                page(path);
            };

            Vue.redirect = function(stateName, routeParams, queryParams) {
                var path = Vue._getPathFromState(states, stateName, routeParams, queryParams);

                page.redirect(path);
            };
        },

        _initDefaultRoute: function() {
            page('*', function() {
                _.nextTick(function() {
                    Vue.redirect(this._defaultRoute);
                }, this);
            }.bind(this));
        },

        _addState: function(state) {
            var url = state.url;

            function onRoute(ctx, next) {
                this._onRoute(state.name, ctx, next);
            }

            page(url, onRoute.bind(this), this._beforeEnter.bind(this), this._updateRoute.bind(this));
            page.exit(url, this._beforeLeave.bind(this));
        },

        _beforeEnter: function(ctx, next) {
            this._callHook('beforeEnter', next);
        },

        _beforeLeave: function(ctx, next) {
            this._callHook('beforeLeave');
            next();
        },

        _onRoute: function(matchType, ctx, next) {
            var queryParams;

            function parseQueryString() {
                queryParams = {};

                ctx.querystring.split('&').map(function(pair) {
                    pair = pair.split('=');
                    queryParams[pair[0]] = pair[1];
                });
            }

            ctx.querystring && parseQueryString();

            this._routerData = this._routesType[matchType];

            if (this._routerData.fresh && this._beforeLocation) {
                document.location.href = ctx.path;
                return;
            }

            this._oldLocation = this._location;

            this._location = {
                path: ctx.path,
                params: ctx.params,
                query: ctx.querystring,
                queryParams: queryParams
            };

            if (this._states) {
                this._location.state = matchType;
            }

            this._beforeLocation = this._location;

            next();
        },

        _updateRoute: function(ctx) {
            var component = this._routerData.component;
            var components = this._root.$options.components;
            var async = this._routerData.async;

            // for webpack
            if (async || this._options.async) {
                require.async(component, function() {
                    this.update(component);
                    this._location.component = component;
                    this._callHook('onEnter');
                }.bind(this));
            } else {
                this.update(component);
                this._location.component = component;
                this._callHook('onEnter');
            }

        },

        _callHook: function(hook, next) {
            var rootMiddleware = this._router.middleware[hook];
            var callback = this._routerData[hook];
            var root = this._root;
            var location = hook === 'beforeLeave' ? null : { to: this._location, from: this._oldLocation };
            var middleware;

            // rootMiddleware一定是function才气作用。
            if (!utils.isFunction(rootMiddleware)) {
                rootMiddleware = null;
            }

            if (callback) {
                if (utils.isFunction(callback)) {
                    middleware = callback;
                } else if (utils.isString(callback)) {
                    if (root[callback]) {
                        middleware = root[callback];
                    }
                }
            }

            _.nextTick(function() {
                root.$emit('router:' + hook, this._location, this._oldLocation, this);
            }, this);

            if (rootMiddleware) {
                rootMiddleware.call(root, location, function() {
                    middleware ? middleware.call(root, location, next, this) : next();
                }.bind(this), this);
            } else if (middleware) {
                middleware.call(root, location, next, this);
            } else if (next) {
                next();
            }
        }
    };
};
